 SUBROUTINE package_parin

!--------------------------------------------------------------------------------!
! This file is part of PALM.
!
! PALM is free software: you can redistribute it and/or modify it under the terms
! of the GNU General Public License as published by the Free Software Foundation,
! either version 3 of the License, or (at your option) any later version.
!
! PALM is distributed in the hope that it will be useful, but WITHOUT ANY
! WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
! A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
!
! You should have received a copy of the GNU General Public License along with
! PALM. If not, see <http://www.gnu.org/licenses/>.
!
! Copyright 1997-2014 Leibniz Universitaet Hannover
!--------------------------------------------------------------------------------!
!
! Current revisions:
! -----------------
!
! Former revisions:
! -----------------
! $Id: package_parin.f90 1325 2014-03-21 09:21:15Z suehring $
!
! 1324 2014-03-21 09:13:16Z suehring
! Bugfix: Missing variable dt_data_output output added to ONLY statement
!
! 1320 2014-03-20 08:40:49Z raasch
! ONLY-attribute added to USE-statements,
! kind-parameters added to all INTEGER and REAL declaration statements,
! kinds are defined in new module kinds,
! old module precision_kind is removed,
! revision history before 2012 removed,
! comment fields (!:) to be used for variable explanations added to
! all variable declaration statements
!
! 1036 2012-10-22 13:43:42Z raasch
! code put under GPL (PALM 3.9)
!
! 828 2012-02-21 12:00:36Z raasch
! +dissipation_classes, radius_classes in parpar
!
! 825 2012-02-19 03:03:44Z raasch
! wang_collision_kernel and turbulence_effects_on_collision in particles_par
! replaced by collision_kernel
!
! Revision 1.1  2000/12/28 13:21:57  raasch
! Initial revision
!
!
! Description:
! ------------
! This subroutine reads from the NAMELIST file variables controling model
! software packages which are used optionally in the run.
!------------------------------------------------------------------------------!

    USE control_parameters,                                                    &
        ONLY:  averaging_interval_sp, dt_data_output, dt_dopts, dt_dosp,       &
               dt_dvrp, particle_maximum_age, skip_time_dosp, threshold

    USE dvrp_variables,                                                        &
        ONLY:  clip_dvrp_l, clip_dvrp_n, clip_dvrp_r, clip_dvrp_s,             &
               cluster_size, color_interval, dvrpsize_interval,                &
               dvrp_directory, dvrp_file, dvrp_host, dvrp_output,              &
               dvrp_password, dvrp_username, groundplate_color,                &
               isosurface_color, mode_dvrp, particle_color,                    &
               particle_dvrpsize, pathlines_fadeintime,                        &
               pathlines_fadeouttime, pathlines_linecount,                     &
               pathlines_maxhistory, pathlines_wavecount,                      &
               pathlines_wavetime, slicer_range_limits_dvrp, superelevation,   &
               superelevation_x, superelevation_y, topography_color,           &
               vc_alpha, vc_gradient_normals, vc_mode, vc_size_x, vc_size_y,   &
               vc_size_z

    USE particle_attributes,                                                   &
        ONLY:  bc_par_b, bc_par_lr, bc_par_ns, bc_par_t, collision_kernel,     &
               density_ratio, dissipation_classes, dt_min_part, dt_prel,       &
               dt_sort_particles, dt_write_particle_data, dvrp_psize,          &
               end_time_prel, initial_weighting_factor,                        &
               maximum_number_of_particles, maximum_number_of_tailpoints,      &
               maximum_tailpoint_age, minimum_tailpoint_distance,              &
               number_of_particle_groups, particles_per_point,                 &
               particle_advection, particle_advection_start, pdx, pdy, pdz,    &
               psb, psl, psn, psr, pss, pst, radius, radius_classes,           &
               random_start_position, read_particles_from_restartfile,         &
               skip_particles_for_tail, use_particle_tails,                    &
               use_sgs_for_particles, vertical_particle_advection,             &
               write_particle_statistics

    USE spectrum,                                                              &
        ONLY:  comp_spectra_level, data_output_sp, plot_spectra_level,         &
               spectra_direction

    IMPLICIT NONE

    CHARACTER (LEN=80) ::  line  !:

    NAMELIST /dvrp_graphics_par/  clip_dvrp_l, clip_dvrp_n, clip_dvrp_r,       &
                                  clip_dvrp_s, cluster_size, color_interval,   &
                                  dt_dvrp, dvrpsize_interval, dvrp_directory,  &
                                  dvrp_file, dvrp_host, dvrp_output,           &
                                  dvrp_password, dvrp_psize, dvrp_username,    &
                                  groundplate_color, isosurface_color,         &
                                  mode_dvrp, particle_color, particle_dvrpsize,&
                                  pathlines_fadeintime, pathlines_fadeouttime, &
                                  pathlines_linecount, pathlines_maxhistory,   &
                                  pathlines_wavecount, pathlines_wavetime,     &
                                  slicer_range_limits_dvrp, superelevation,    &
                                  superelevation_x, superelevation_y,          &
                                  threshold, topography_color, vc_alpha,       &
                                  vc_gradient_normals, vc_mode, vc_size_x,     &
                                  vc_size_y, vc_size_z

    NAMELIST /particles_par/      bc_par_b, bc_par_lr, bc_par_ns, bc_par_t,    &
                                  collision_kernel, density_ratio,             &
                                  dissipation_classes, dt_dopts,               &
                                  dt_min_part, dt_prel, dt_sort_particles,     &
                                  dt_write_particle_data,                      &
                                  end_time_prel, initial_weighting_factor,     &
                                  maximum_number_of_particles,                 &
                                  maximum_number_of_tailpoints,                &
                                  maximum_tailpoint_age,                       &
                                  minimum_tailpoint_distance,                  &
                                  number_of_particle_groups,                   &
                                  particles_per_point,                         &
                                  particle_advection_start,                    &
                                  particle_maximum_age, pdx, pdy, pdz, psb,    &
                                  psl, psn, psr, pss, pst, radius,             &
                                  radius_classes, random_start_position,       &
                                  read_particles_from_restartfile,             &
                                  skip_particles_for_tail,                     &
                                  use_particle_tails, use_sgs_for_particles,   &
                                  vertical_particle_advection,                 &
                                  write_particle_statistics

    NAMELIST /spectra_par/        averaging_interval_sp, comp_spectra_level,   &
                                  data_output_sp, dt_dosp, plot_spectra_level, &
                                  skip_time_dosp, spectra_direction

!
!-- Position the namelist-file at the beginning (it was already opened in
!-- parin), search for the namelist-group of the package and position the
!-- file at this line. Do the same for each optionally used package.
    line = ' '

#if defined( __dvrp_graphics )
    REWIND ( 11 )
    line = ' '
    DO   WHILE ( INDEX( line, '&dvrp_graphics_par' ) == 0 )
       READ ( 11, '(A)', END=10 )  line
    ENDDO
    BACKSPACE ( 11 )

!
!-- Read user-defined namelist
    READ ( 11, dvrp_graphics_par )

 10 CONTINUE
#endif

!
!-- Try to find particles package
    REWIND ( 11 )
    line = ' '
    DO   WHILE ( INDEX( line, '&particles_par' ) == 0 )
       READ ( 11, '(A)', END=20 )  line
    ENDDO
    BACKSPACE ( 11 )

!
!-- Read user-defined namelist
    READ ( 11, particles_par )

!
!-- Set flag that indicates that particles are switched on
    particle_advection = .TRUE.

 20 CONTINUE


#if defined( __spectra )
    REWIND ( 11 )
    line = ' '
    DO   WHILE ( INDEX( line, '&spectra_par' ) == 0 )
       READ ( 11, '(A)', END=30 )  line
    ENDDO
    BACKSPACE ( 11 )

!
!-- Read user-defined namelist
    READ ( 11, spectra_par )

!
!-- Default setting of dt_dosp here (instead of check_parameters), because its
!-- current value is needed in init_pegrid
    IF ( dt_dosp == 9999999.9 )  dt_dosp = dt_data_output

 30 CONTINUE
#endif

 END SUBROUTINE package_parin
